@@ -1,42 +1,31 @@
-"""
-Plugin support for rdf.
+"""Plugin support for rdf.
 
 There are a number of plugin points for rdf: parser, serializer,
 store, query processor, and query result. Plugins can be registered
 either through setuptools entry_points or by calling
 rdf.plugin.register directly.
 
-If you have a package that uses a setuptools based setup.py you can add the
-following to your setup::
+If you have a package that uses a setuptools based `setup.py` you can add the
+following to your setup:
 
-    entry_points = {
-        'rdf.plugins.parser': [
-            'nt =     rdf.plugins.parsers.ntriples:NTParser',
-            ],
-        'rdf.plugins.serializer': [
-            'nt =     rdf.plugins.serializers.NTSerializer:NTSerializer',
-            ],
-        }
-
-See the `setuptools dynamic discovery of services and plugins`__ for more
-information.
-
-.. __: http://peak.telecommunity.com/DevCenter/setuptools#dynamic-discovery-of-services-and-plugins
+```python
+entry_points = {
+    'rdf.plugins.parser': [
+        'nt =     rdf.plugins.parsers.ntriples:NTParser',
+        ],
+    'rdf.plugins.serializer': [
+        'nt =     rdf.plugins.serializers.NTSerializer:NTSerializer',
+        ],
+    }
+```
 
+See the [setuptools dynamic discovery of services and plugins](http://peak.telecommunity.com/DevCenter/setuptools#dynamic-discovery-of-services-and-plugins)
+for moreinformation.
 """
 
-from rdflib.store import Store
-from rdflib.parser import Parser
-from rdflib.serializer import Serializer
-from rdflib.query import (
-    ResultParser,
-    ResultSerializer,
-    Processor,
-    Result,
-    UpdateProcessor,
-)
-from rdflib.exceptions import Error
-import sys
+from __future__ import annotations
+
+from importlib.metadata import EntryPoint, entry_points
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -50,10 +39,28 @@ from typing import (
     overload,
 )
 
-if TYPE_CHECKING:
-    from pkg_resources import EntryPoint
+import rdflib.plugins.stores.berkeleydb
+from rdflib.exceptions import Error
+from rdflib.parser import Parser
+from rdflib.query import (
+    Processor,
+    Result,
+    ResultParser,
+    ResultSerializer,
+    UpdateProcessor,
+)
+from rdflib.serializer import Serializer
+from rdflib.store import Store
 
-__all__ = ["register", "get", "plugins", "PluginException", "Plugin", "PKGPlugin"]
+__all__ = [
+    "register",
+    "get",
+    "plugins",
+    "PluginException",
+    "Plugin",
+    "PluginT",
+    "PKGPlugin",
+]
 
 rdflib_entry_points = {
     "rdf.plugins.store": Store,
@@ -66,13 +73,14 @@ rdflib_entry_points = {
     "rdf.plugins.updateprocessor": UpdateProcessor,
 }
 
-_plugins: Dict[Tuple[str, Type[Any]], "Plugin"] = {}
+_plugins: Dict[Tuple[str, Type[Any]], Plugin] = {}
 
 
-class PluginException(Error):
+class PluginException(Error):  # noqa: N818
     pass
 
 
+#: A generic type variable for plugins
 PluginT = TypeVar("PluginT")
 
 
@@ -86,7 +94,7 @@ class Plugin(Generic[PluginT]):
         self.class_name = class_name
         self._class: Optional[Type[PluginT]] = None
 
-    def getClass(self) -> Type[PluginT]:
+    def getClass(self) -> Type[PluginT]:  # noqa: N802
         if self._class is None:
             module = __import__(self.module_path, globals(), locals(), [""])
             self._class = getattr(module, self.class_name)
@@ -94,13 +102,13 @@ class Plugin(Generic[PluginT]):
 
 
 class PKGPlugin(Plugin[PluginT]):
-    def __init__(self, name: str, kind: Type[PluginT], ep: "EntryPoint"):
+    def __init__(self, name: str, kind: Type[PluginT], ep: EntryPoint):
         self.name = name
         self.kind = kind
         self.ep = ep
         self._class: Optional[Type[PluginT]] = None
 
-    def getClass(self) -> Type[PluginT]:
+    def getClass(self) -> Type[PluginT]:  # noqa: N802
         if self._class is None:
             self._class = self.ep.load()
         return self._class
@@ -127,11 +135,6 @@ def get(name: str, kind: Type[PluginT]) -> Type[PluginT]:
     return p.getClass()
 
 
-if sys.version_info < (3, 8):
-    from importlib_metadata import entry_points
-else:
-    from importlib.metadata import entry_points
-
 all_entry_points = entry_points()
 if hasattr(all_entry_points, "select"):
     for entry_point, kind in rdflib_entry_points.items():
@@ -139,26 +142,26 @@ if hasattr(all_entry_points, "select"):
             _plugins[(ep.name, kind)] = PKGPlugin(ep.name, kind, ep)
 else:
     # Prior to Python 3.10, this returns a dict instead of the selection interface, which is slightly slower
+    if TYPE_CHECKING:
+        assert isinstance(all_entry_points, dict)
     for entry_point, kind in rdflib_entry_points.items():
-        for ep in all_entry_points.get(entry_point, []):  # type: ignore[union-attr]
+        for ep in all_entry_points.get(entry_point, []):
             _plugins[(ep.name, kind)] = PKGPlugin(ep.name, kind, ep)
 
 
 @overload
 def plugins(
     name: Optional[str] = ..., kind: Type[PluginT] = ...
-) -> Iterator[Plugin[PluginT]]:
-    ...
+) -> Iterator[Plugin[PluginT]]: ...
 
 
 @overload
-def plugins(name: Optional[str] = ..., kind: None = ...) -> Iterator[Plugin]:
-    ...
+def plugins(name: Optional[str] = ..., kind: None = ...) -> Iterator[Plugin]: ...
 
 
 def plugins(
     name: Optional[str] = None, kind: Optional[Type[PluginT]] = None
-) -> Iterator[Plugin]:
+) -> Iterator[Plugin[PluginT]]:
     """
     A generator of the plugins.
 
@@ -170,6 +173,15 @@ def plugins(
 
 
 # Register Stores
+
+if rdflib.plugins.stores.berkeleydb.has_bsddb:
+    # Checks for BerkeleyDB before registering it
+    register(
+        "BerkeleyDB",
+        Store,
+        "rdflib.plugins.stores.berkeleydb",
+        "BerkeleyDB",
+    )
 register(
     "default",
     Store,
@@ -200,12 +212,7 @@ register(
     "rdflib.plugins.stores.concurrent",
     "ConcurrentStore",
 )
-register(
-    "BerkeleyDB",
-    Store,
-    "rdflib.plugins.stores.berkeleydb",
-    "BerkeleyDB",
-)
+
 register(
     "SPARQLStore",
     Store,
@@ -354,6 +361,12 @@ register(
     "rdflib.plugins.serializers.hext",
     "HextuplesSerializer",
 )
+register(
+    "patch",
+    Serializer,
+    "rdflib.plugins.serializers.patch",
+    "PatchSerializer",
+)
 
 # Register Triple Parsers
 register(
@@ -479,6 +492,14 @@ register(
     "HextuplesParser",
 )
 
+# Register RDF Patch Parsers
+register(
+    "patch",
+    Parser,
+    "rdflib.plugins.parsers.patch",
+    "RDFPatchParser",
+)
+
 # Register SPARQL Processors
 register(
     "sparql",
@@ -556,18 +577,6 @@ register(
     "rdflib.plugins.sparql.results.xmlresults",
     "XMLResultParser",
 )
-register(
-    "application/sparql-results+xml; charset=UTF-8",
-    ResultParser,
-    "rdflib.plugins.sparql.results.xmlresults",
-    "XMLResultParser",
-)
-register(
-    "application/rdf+xml",
-    ResultParser,
-    "rdflib.plugins.sparql.results.graph",
-    "GraphResultParser",
-)
 register(
     "json",
     ResultParser,
@@ -604,3 +613,14 @@ register(
     "rdflib.plugins.sparql.results.tsvresults",
     "TSVResultParser",
 )
+
+graph_parsers = {parser.name for parser in plugins(kind=Parser)}
+result_parsers = {parser.name for parser in plugins(kind=ResultParser)}
+graph_result_parsers = graph_parsers - result_parsers
+for parser_name in graph_result_parsers:
+    register(
+        parser_name,
+        ResultParser,
+        "rdflib.plugins.sparql.results.graph",
+        "GraphResultParser",
+    )
